
/***********************************************************************************
 Mega2560

use the BREAKOUT BOARD only and use these 8bit data lines to the LCD,
pin usage:
                  LCD_CS  LCD_CD  LCD_WR  LCD_RD  LCD_RST  SD_SS  SD_DI  SD_DO  SD_SCK 
Arduino Mega2560    A3      A2      A1      A0      A4      10     11     12      13                           

**********************************************************************************/

/**
RAPPEL ATmega 2560 :
– 256KBytes of In-System Self-PrograM2able Flash
– 4Kbytes EEPROM
– 8Kbytes Internal SRAM
**/

#include "Arduino.h"
#include <stdint.h>

#include <LCDWIKI_GUI.h> //Core graphics library
#include <LCDWIKI_KBV.h> //Hardware-specific library


#include "Graphiques_TFT.h"
	
/**  Graphiques_TFT **/

String version="3.0";


int sram;
uint32_t bmp_offset = 0;
uint16_t s_width = TFT480.Get_Display_Width();  
uint16_t s_heigh = TFT480.Get_Display_Height();

#define NOIR   0x0000
#define ROUGE  0xF800
#define ORANGE 0xFBC0
#define JAUNE   0xFFE0
#define VERT   0x07E0
#define CYAN    0x07FF
#define BLEU_CLAIR   0x455F
#define BLEU   0x001F
#define MAGENTA 0xF81F
#define VIOLET	0x781A
#define GRIS_CLAIR  0xA534
#define GRIS   			0x8410
#define GRIS_FONCE  0x5ACB
#define BLANC   0xFFFF



const int INPUT_PIN = 22;
uint8_t SDcardOk=0;



    
/** -------------------------------------------------------
  SD card
----------------------------------------------------------- 
ADAPT SD	-> mega2560 pins

	CS			->  53
	MISO		->  50
	MOSI		->  51
	SCK			->  52 
**/


int freeRam()  // pour tests
{
  extern int __heap_start, *__brkval;
  int v;
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval);
}



/**--------------------------------------------------------
	fonctions AFFICHAGE TFT
--------------------------------------------------------**/    


void clear_screen()
{
  TFT480.Fill_Screen(40,40,40);  
}


void dessine_triangles()
{
   int i = 0;
   uint16_t L, H;
   L=TFT480.Get_Display_Width();
   H=TFT480.Get_Display_Height();
   
   for(i=0; i<H/2; i+=5)
   {
      TFT480.Set_Draw_color(0,i+64,i+64);
      TFT480.Draw_Triangle(L/2-1,H/2-1-i,L/2-1-i,H/2-1+i,L/2-1+i,H/2-1+i);                   
   }
}


void dessine_degrade(uint16_t x, uint16_t y, uint8_t longueur, uint8_t RR1, uint8_t RR2, uint8_t GG1, uint8_t GG2, uint8_t BB1, uint8_t BB2)
{
// dessine un dégr&dé horizontal de couleurs (R1,G1,B1) -> (R2,G2,B2)	
	uint16_t i;
	float R,G,B;

	if(longueur==0) {return;}

	i=0;
	
	R=RR1;
	G=GG1;
	B=BB1;
	
	while(i<longueur)
	{
		x++;
		R+=(RR2-RR1)/longueur;
		G+=(GG2-GG1)/longueur;
		B+=(BB2-BB1)/longueur;
		TFT480.Set_Draw_color((uint8_t)R,(uint8_t)G,(uint8_t)B);
		TFT480.Draw_Line(x, y, x, y+10);
		i++;
	}
}


void dessine_arc_en_ciel()
{
	int16_t i;
	float xr,xv,xb;
	float r,v,b;
	for (i=0; i<63; i++) // 10 * 2 pi radians
	{
		xr=(i+12)/10.0;		r=140.0+120.0*sin(xr)-i; if (r<0) {r=0;}  if (r>255) {r=255;}
		xv=(i+54)/10.0;		v=130.0+150.0*sin(xv); if (v<0) {v=0;}  if (v>255) {v=255;};
		xb=(i+38)/10.0;		b=100.0+150.0*sin(xb); if (b<0) {b=0;}  if (b>255) {b=255;};

/*
//pour test du déphasage des courbes sinus...
		TFT480.Set_Draw_color(255, 255, 255);
		TFT480.Draw_Pixel(200+i, 150-(uint8_t)(r/10.0));
		TFT480.Draw_Pixel(200+i, 200-(uint8_t)(v/10.0));
		TFT480.Draw_Pixel(200+i, 250-(uint8_t)(b/10.0));
*/

		TFT480.Set_Draw_color((uint8_t)r,(uint8_t)v,(uint8_t)b);
		TFT480.Draw_Circle_Helper(400, 280, 400-i, 1);
	}
}


void dessine_cercle(uint16_t x0, uint16_t y0, uint16_t r, uint16_t couleur)
{
	uint16_t n;
	float i;
	float x,y;

	TFT480.Set_Draw_color(couleur);
	i=0;
	while(i<2*M_PI)
	{
		x=x0+r*cos(i);
		y=y0+r*sin(i);
		TFT480.Draw_Pixel(x,y);
		i+=0.01;  // radians
	}
}


void dessine_ellipse(uint16_t x0, uint16_t y0, int16_t dx, int16_t dy, uint16_t couleur)
{
	uint16_t n;
	float i;
	float x,y;

	TFT480.Set_Draw_color(couleur);
	i=0;
	while(i<2*M_PI)
	{
		x=x0+dx*cos(i);
		y=y0+dy*sin(i);
		TFT480.Draw_Pixel(x,y);
		i+=0.01;  // radians
	}
}


void dessine_ellipse_inclinee()
{
	uint16_t n;
	float i;
	float x,y;

	TFT480.Set_Draw_color(VERT);
	i=0;
	while(i<2*M_PI)
	{
		x=280+50.0*cos(i-0.5);
		y=150+80.0*cos(i+0.5);
		TFT480.Draw_Pixel(x,y);
		i+=0.01;  // radians
	}
}



 
void Draw_arc_elliptique(uint16_t x0, uint16_t y0, int16_t dx, int16_t dy, float alpha1, float alpha2, uint16_t couleur) // alpha1 et alpha2 en radians
{
/*
REMARQUES :	
-cette fonction permet également de dessiner un arc de cercle (si dx=dy), voire le cercle complet
- dx et dy sont du type int (et pas uint) et peuvent êtres négafifs, ou nuls.
-alpha1 et alpha2 sont les angles (en radians) des azimuts des extrémités de l'arc 
*/ 
	uint16_t n;
	float i;
	float x,y;

	TFT480.Set_Draw_color(couleur);
	i=alpha1;
	while(i<alpha2)
	{
		x=x0+dx*cos(i);
		y=y0+dy*cos(i+M_PI/2.0);
		TFT480.Draw_Pixel(x,y);
		i+=0.01;  // radians
	}
}


void Fill_croissant_elliptique(uint16_t x0, uint16_t y0, int16_t dx1, int16_t dx2, int16_t dy, float alpha1, float alpha2, uint16_t couleur)
{
	// dx1 et dx2 sont les extremums (en pixels) horizontaux d'un croissant vertical

	if (dx2<dx1) {return;}
	int16_t dx;
	dx = dx1;
	while(dx<dx2)
	{
		Draw_arc_elliptique(x0, y0, dx, dy, alpha1, alpha2, couleur);
		dx++;
	}
}





void dessine_phases_Lune()  // animation
{
//très bien pour une animation, mais pas pratique pour dessiner une phase donnée en Lune descendante	
	float i;
	uint16_t x,y;
	uint16_t n;

//Lune montante
	for(n=0 ; n< 200; n++)
	{
		x = random(470);
		y = random(300);
		if (((x-200)*(x-200) +(y-150)*(y-150)) > 5000)
		{
			TFT480.Set_Draw_color(BLANC); // étoiles
			TFT480.Draw_Pixel(x,y);
		}	
	}
	
	Draw_arc_elliptique(200, 150, 60, 60, 0, 2*M_PI, GRIS_FONCE); // cercle complet

	i= 60;
	while(i>(-60))
	{
		Draw_arc_elliptique(200, 150, i, 60, -M_PI/2, M_PI/2, BLANC);
		i-=1;
		delay(10);
	}

	Draw_arc_elliptique(200, 165, 30, -20, 0, M_PI, GRIS_FONCE);
	dessine_cercle(180,140,5, NOIR);
	dessine_cercle(220,140,5, NOIR);


//Lune descendante
	i= 60;
	while(i>(-60))
	{
		Draw_arc_elliptique(200, 150, i, 60, -M_PI/2, M_PI/2, GRIS_FONCE); // dessine en noir par dessus la pleine Lune
		i-=1;
		delay(10);
	}
}



void Dessine_Lune(uint16_t x0, uint16_t y0,	uint8_t r, float age)
{
//Remarque : l'age admet des nombres réels (avec décimales)

	int16_t T1, T2; //Terminators gauche et droite, limites entre partie éclairée et partie dans l'ombre

	if (age<1) {dessine_cercle(x0, y0, r, BLANC);} // Nouvelle Lune
	if (age <= 15) // premier croissant... pleine
	{
		T1=r-2*r*age/15;
		T2=r;
	}
	if ((age > 15) && (age <=30)) // pleine... dernier croissant
	{
		T1=-r;
		T2=r-2*r*(age-15)/15;
	}
	Fill_croissant_elliptique(x0, y0, T1, T2, r,  -M_PI/2, M_PI/2, BLANC);
}



float Decimale(float valeur)
{
	float d;
	d=valeur-floor(valeur); 
	return d;
}


float GetPhase(int Y, int M, int D)
{
// return "age de le Lune" (nombre allant de 0.00 à 29.xx)
	float AG, IP;
	uint32_t Y2, M2, K1, K2, K3, JD;
	Y2=Y-floor((12-M)/10);
	M2=M+9;
	if (M2>=12)	{M2-=12;}
	K1= floor(365.25 * (Y2 + 4712));
	K2= floor(30.6 * M2+0.5);
	K3= floor(floor((Y2/100) + 49)*0.75)-38;
	JD= K1+K2+D+59;
	if (JD > 2299160)	{JD=JD-K3;}
	IP= Decimale((JD-2451550.1)/29.53058);
	AG= IP*29.53;
	return AG;
}





void setup()
{
	Serial.begin(115200);

	TFT480.Init_LCD();
	clear_screen();
	delay(10);

	TFT480.Set_Rotation(1);

	TFT480.Set_Text_Back_colour(NOIR);
	TFT480.Set_Text_colour(BLANC);
	TFT480.Set_Text_Size(2);
	TFT480.Print_String("Plein de choses utiles", 0, 10);
	TFT480.Print_String("pour une Station Meteo", 0, 30);
	
	delay(2000);
	clear_screen();
	
// =======================================================================================	
// *********  DEGRADES DE COULEUR **************

	uint8_t RR1, RR2, GG1, GG2, BB1, BB2;

//primaires noir -> clair
	RR1=0;	RR2=255;
	GG1=0;	GG2=0;
	BB1=0;	BB2=0;
	dessine_degrade(0, 0, 255, RR1, RR2, GG1, GG2, BB1, BB2);

	RR1=0;	RR2=0;
	GG1=0;	GG2=255;
	BB1=0;	BB2=0;
	dessine_degrade(0, 20, 255, RR1, RR2, GG1, GG2, BB1, BB2);

	RR1=0;	RR2=0;
	GG1=0;	GG2=0;
	BB1=0;	BB2=255;
	dessine_degrade(0, 40, 255, RR1, RR2, GG1, GG2, BB1, BB2);

//primaires clair -> noir
	RR1=255;	RR2=0;
	GG1=0;	GG2=0;
	BB1=0;	BB2=0;
	dessine_degrade(0, 60, 255, RR1, RR2, GG1, GG2, BB1, BB2);

	RR1=0;	RR2=0;
	GG1=255;	GG2=0;
	BB1=0;	BB2=0;
	dessine_degrade(0, 80, 255, RR1, RR2, GG1, GG2, BB1, BB2);

	RR1=0;	RR2=0;
	GG1=0;	GG2=0;
	BB1=255;	BB2=0;
	dessine_degrade(0, 100, 255, RR1, RR2, GG1, GG2, BB1, BB2);

// rouge ->vert
	RR1=255;	RR2=0;
	GG1=0;	GG2=255;
	BB1=0;	BB2=0;
	dessine_degrade(0, 120, 255, RR1, RR2, GG1, GG2, BB1, BB2);

// rouge ->vert très court
	RR1=255;	RR2=0;
	GG1=0;	GG2=255;
	BB1=0;	BB2=0;
	dessine_degrade(300, 120, 37, RR1, RR2, GG1, GG2, BB1, BB2);	

// jaune -> noir
	RR1=255;	RR2=0;
	GG1=255;	GG2=0;
	BB1=0;	BB2=0;
	dessine_degrade(0, 140, 255, RR1, RR2, GG1, GG2, BB1, BB2);

	
// jaune -> bleu
	RR1=255;	RR2=0;
	GG1=255;	GG2=0;
	BB1=0;	BB2=255;
	dessine_degrade(0, 160, 255, RR1, RR2, GG1, GG2, BB1, BB2);

// jaune -> bleu en plus court
	RR1=255;	RR2=0;
	GG1=255;	GG2=0;
	BB1=0;	BB2=255;
	dessine_degrade(300, 160, 100, RR1, RR2, GG1, GG2, BB1, BB2);	
	
// noir -> blanc
	RR1=0;	RR2=255;
	GG1=0;	GG2=255;
	BB1=0;	BB2=255;
	dessine_degrade(0, 180, 255, RR1, RR2, GG1, GG2, BB1, BB2);

// noir -> cyan
	RR1=0;	RR2=0;
	GG1=0;	GG2=255;
	BB1=0;	BB2=255;
	dessine_degrade(0, 200, 255, RR1, RR2, GG1, GG2, BB1, BB2);

// rouge -> cyan
	RR1=255;	RR2=0;
	GG1=0;	GG2=255;
	BB1=0;	BB2=255;
	dessine_degrade(0, 220, 255, RR1, RR2, GG1, GG2, BB1, BB2);

// orange -> bleu
	RR1=255;	RR2=0;
	GG1=120;	GG2=0;
	BB1=0;	BB2=255;
	dessine_degrade(0, 240, 255, RR1, RR2, GG1, GG2, BB1, BB2);


// orange -> bleu plus court
	RR1=255;	RR2=0;
	GG1=120;	GG2=0;
	BB1=0;	BB2=255;
	dessine_degrade(300, 240, 85, RR1, RR2, GG1, GG2, BB1, BB2);
	

//longueur variable
	for(uint8_t n=10 ; n<255 ; n++)
	{
		RR1=255;	RR2=0;
		GG1=0;	GG2=255;
		BB1=0;	BB2=0;
	//	dessine_degrade(0, 260, n, RR1, RR2, GG1, GG2, BB1, BB2);	
	}

// =======================================================================================
	delay(1000);
	clear_screen();

	TFT480.Set_Text_Back_colour(NOIR);
	TFT480.Set_Text_colour(BLANC);
	TFT480.Set_Text_Size(1);
	TFT480.Print_String("Affichage d'ellipses parametriques en cos et sin", 30, 10);



	double phase;
	int j;
	
	for (j=1; j<31; j++)
	{ 
		phase = GetPhase(2020,11,j);
		Serial.print(j); Serial.print(" Phase="); Serial.println(phase);
	}

	delay(1000);

	clear_screen();
	dessine_arc_en_ciel();

	dessine_cercle(100,150,80, JAUNE); 
	dessine_ellipse(100,150,60,120, ORANGE);
	dessine_ellipse_inclinee();
	Draw_arc_elliptique(410, 150, 60, 100, -M_PI/2.0, M_PI/2.0, JAUNE );
	Draw_arc_elliptique(405, 150, 60, 60, 0, 2*M_PI, CYAN); // <- dessin d'un cercle

	delay(1000);
	clear_screen();

	dessine_phases_Lune();

	delay(1000);
	clear_screen();

// =======================================================================================	

//RAPPEL : Fill_croissant_elliptique(x0, y0, dx1, dx2, dy, alpha1, alpha2, couleur)

	Fill_croissant_elliptique(10, 150, 50, 60, 60,  -M_PI/2, M_PI/2, BLANC); // premier croissant
	Fill_croissant_elliptique(100, 150, 0, 60, 60,  -M_PI/2, M_PI/2, BLANC); // premier quartier	
	Fill_croissant_elliptique(230, 150, -60, 60, 60,  -M_PI/2, M_PI/2, BLANC); // pleine
	Fill_croissant_elliptique(360, 150, -60, 30, 60,  -M_PI/2, M_PI/2, BLANC); //Gibi
	Fill_croissant_elliptique(460, 150, -60, -40, 60,  -M_PI/2, M_PI/2, BLANC);// dernier croissant

	Fill_croissant_elliptique(250, 240, -20, 10, 20,  -M_PI/2, M_PI/2, BLANC); // + petite


	delay(2000);
	clear_screen();

// =======================================================================================


// Dessine une suite de Lunes par une formule unique en fonction de l'age de la Lune
// non, non, il ne s'agit pas de son age depuis sa naissance il y a 4.53 milliards d'années !!!
// L'âge de la lune est le nombre de jours écoulés depuis la nouvelle lune : 0 le jour de la nouvelle lune, 1 le lendemain ... 

	TFT480.Set_Text_Back_colour(NOIR);
	TFT480.Set_Text_colour(BLANC);
	TFT480.Set_Text_Size(2);
	TFT480.Print_String(" 'Age' de la Lune (en jours)", 10, 10);
	
	uint8_t age =0;
	uint8_t r = 10;
	
	TFT480.Set_Text_Size(1);
	TFT480.Set_Text_colour(VERT);
	while(age <= 15)
	{
		TFT480.Print_Number_Int(age,20+22*age, 58, 2, ' ', 10);
		Dessine_Lune(20+22*age, 80,	r, age);
		age++;
	}
	while ((age > 15) && (age < 30) )
	{
		TFT480.Print_Number_Int(age, -300+20*age, 108, 2, ' ', 10);
		Dessine_Lune(-300 +8+20*age, 130,	r, age); // 2eme ligne + bas
		age++;
	}
}

    
void loop()
{
	delay(10);
}




