
#ifndef STATION_METEO_H
#define STATION_METEO_H

#include <stdint.h>
#include "Arduino.h"

#include <LCDWIKI_GUI.h> //Core graphics library
#include <LCDWIKI_KBV.h> //Hardware-specific library

#include <Wire.h>
#include <TimeLib.h>
#include <DS1307RTC.h>

#include <SPI.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BMP280.h>

#include <SD.h>

	

//if the IC model is known or the modules is unreadable,you can use this constructed function
LCDWIKI_KBV TFT480(ILI9486,A3,A2,A1,A0,A4); //model,cs,cd,wr,rd,reset

//if the IC model is not known and the modules is readable,you can use this constructed function
//LCDWIKI_KBV TFT480(320,480,A3,A2,A1,A0,A4);//width,height,cs,cd,wr,rd,reset

/** ***********************************************************************************
		CLASS Etiq3  // affiche un nombre ou un petit texte dans un rectangle						
***************************************************************************************/



class Etiq3
{
public:

	uint16_t x0; //position
	uint16_t y0;
	uint16_t dx; //dimension
	uint16_t dy;

//couleurs
	uint16_t couleur_cadre;
	uint16_t couleur_fond;
	uint16_t couleur_nom;
	uint16_t couleur_txt;

	
	Etiq3();
	
	void init(uint16_t x, uint16_t y, uint16_t dx, uint16_t dy,char nom[10]);
	void setCouleurCadre(uint16_t couleur_i);
	void setCouleurNom(uint16_t couleur_i);
	void setCouleurTxt(uint16_t couleur_i);
	void setCouleurFond(uint16_t couleur_i);
	void efface();
	void flashFond(uint16_t couleur_i);
	void affiche_int(uint32_t valeur, uint8_t nb_chiffres, char txt_unite_i[3], uint32_t v_min, uint32_t v_max);
	void affiche_float(float valeur, uint8_t nb_chiffres, uint8_t nb_dec, char txt_unite_i[3], float v_min, float v_max);
	void affiche_HEXA(uint32_t valeur);
	void affiche_texte(char txt_i[5]);
	void affiche_string(String txt_i);
	
	
private:
	char nom[13];
	char txt_unite[4];
	char texte[11];

	void traceCadre();
	void affi_nom();

};


/** ***********************************************************************************
		CLASS Scope  // affiche des courbes dans une surface rectangulaire
************************************************************************************* */

class Scope
{
public:

	uint16_t x0; //position
	uint16_t y0;
	uint16_t dx; //dimension
	uint16_t dy;

//couleurs
	uint16_t couleur_trace;
	uint16_t couleur_cadre;
	
	Scope();
	
	void init(uint16_t x, uint16_t y, uint16_t dx, uint16_t dy);
	void setCouleurCadre(uint16_t couleur_i);
	void setCouleurTrace(uint16_t couleur_i);
	void traceCadre();
	void efface_surface();
	void efface_libel_gradu();
	void traceGraduation();
	void Plot(uint16_t x, uint16_t y);
	void Cercle(uint16_t x1, uint16_t y1, uint16_t r1,  uint16_t couleur_i);
	void Tiret(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
	
	
private:


};


class LED
{
public:

	uint16_t x0; //position
	uint16_t y0;
	uint16_t dx; //dimension
	uint16_t dy;

	
	LED();
	
	void init(uint16_t xi, uint16_t yi, uint16_t dxi, uint16_t dyi);
	void setCouleur(uint16_t couleur_i);
	
	
private:


};

/************************************************************************************** */
// AUTRES DECLARATIONS


struct record_meteo
{
	int16_t T; // température
	int16_t P; // pression
	uint8_t H;  // humidité
// ligne ci-dessous : 2 infos multiplexées pour économiser de la SRAM (sinon on passe pas !)	
	uint16_t heure; // au format h*60+mn par exemple pour 23h59 --> 23*60+59=1439 + data invalide si = 0xFFFF
};

int freeRam();

void Init_SDcard();
void init_ports();
void init_variables();
void trace_ecran();
void trace_entete();
void clear_screen();
void dessine_triangles();
void dessine_arc_en_ciel();
void affiche_courbeT();
void affiche_courbeP();
void affiche_courbeH();
void affiche_ligne_ref();
void affiche_pannel();
void affiche_LEDS();

void RGB565_to_888(uint16_t color565, uint8_t *R, uint8_t *G, uint8_t *B);
void draw_bmp(uint16_t x0, uint16_t y0, File fp);
void affi_img(uint16_t x0, uint16_t y0, const char* filename1);

void RAZ_data();
void acqui_PressionTemperature();
void init_lst_records();
void efface_buffer();
void record_enr_actuel();
void decale_enregistrements(uint16_t p);
int16_t strbinToInt(String str1, uint8_t nb_bits);
void lit_temperature();
void calcul_confiance();
uint16_t meilleure_valeur();
void lit_humidite();
void decodeBuffer();
void setup();

void verifie_data_scope();
void efface_glitch_P();

void determine_Tmin_Tmax();
void determine_Pmin_Pmax();

void calcul_Gradu_ech_T();
void calcul_ech_P();

void trace_sur_Scope(); 
void affiche_meteo();

void tt_les_1mn();
void tt_les_5mn();
void tt_les_30mn();

void loop();
void analyse_signal_primaire();
void affiche_date();
void affiche_heure();
void setDateTime();
void print_meteo_on_SDcard(); // en ASCII
void write_scope_on_SDcard();  // en binaire
void read_scope_on_SDcard();
void write_TFT_on_SDcard();

#endif
